-- Simplex G-LINK S-LINK LSC-Radtest Version (TSC) 
-- Created on 17. Nov. 1998
-- Developer: Zoltan Meggyesi
-- Pinout and logic options in tsc.  file

-- Last modification: 08.12.98
-- Change made: First version

------------------------------------------------------------------------------

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_ARITH.ALL;

entity tsc is

  port(
        AC_CLKA           : in    std_logic;  -- 40 Mhz from Quartz
	AC_CLKA_IO        : in    std_logic;  -- Quartz CLK to IO pin
--	UCLK              : in    std_logic;  -- S-LINK UCLK
--	UCLK_IO           : in    std_logic;  -- S-LINK UCLK to IO pin	

--     	ad_oti            : in    std_logic;  -- A/D converter pins
--      ad_dio            : in    std_logic;  -- Not used in this version

        LINE_P0           : in    std_logic;  -- Pattern Select
        LINE_P1           : in    std_logic;  -- Pattern Select
        LINE_RST          : in    std_logic;  -- External reset
	LINE_SIG4         : in    std_logic;  -- Spare external signal

        HP_LOCKED         : in    std_logic;  -- In-Lock indication
	
--	UD                : in    std_logic_vector(31 downto 0);
--      UCTRL             : in    std_logic; 
--      UWEN              : in    std_logic; 

--      URESET            : in    std_logic;
--      UTEST             : in    std_logic;
--      UDW               : in    std_logic_vector(1 downto 0); -- "00" in 32 b mode
	
	FBIN              : in    std_logic_vector(15 downto 0); -- I/O Feedback
        FBOUT             : out   std_logic_vector(15 downto 0); -- I/O Feedback
	
        TRIG	          : out   std_logic;  -- Twisted pair "Trigger" 
	Q1                : out   std_logic;  -- Twisted pair signal 1
	Q2                : out   std_logic;  -- Twisted pair signal 2
	Q3                : out   std_logic; -- Twisted pair signal 3    
	
--      ad_convst         : out   std_logic;  -- Temperature Sensor outputs
--      ad_sclk           : out   std_logic;
--      ad_rdwr           : out   std_logic; 
		    
        HP_D              : out   std_logic_vector(19 downto 0); -- G-LINK Data
	HP_CAV_N          : out   std_logic;    -- G-Link Data Available
	HP_DAV_N          : out   std_logic;    -- G-Link Ctrl Word Available
	
        HP_RST_N          : out   std_logic;  -- HP Chip reset 
	HP_DIV            : out   std_logic_vector(1 downto 0); -- Baud Rate
	HP_FLAGSEL        : out   std_logic;  -- Flag Bit Mode Select
	HP_FLAG           : out   std_logic;  -- Flag bit
        HP_M20SEL         : out   std_logic;  -- 16/20 bit mode
        HP_MDFSEL         : out   std_logic;  -- Double Frame Mode select

--      LFF               : out   std_logic;  
	LDOWN_N           : out   std_logic;  -- Connected for test purpose
--	LRL               : out   std_logic_vector(3 downto 0);

        TESLED_N          : out   std_logic; -- Connected to 4. Line input for test
        LUPLED_N          : out   std_logic; -- Connected for test purpose
--      FLOWCTLED         : out   std_logic;

	AC_CLKABUF        : out   std_logic);  -- CLK for G-LINK chip 

end tsc;

ARCHITECTURE behavioural OF tsc IS

-- Global clock, reset
  signal QCLK, resetsignal, linereset: std_logic;
-- G-Link Data TX signals
  signal data_n, data_n1,  writen, ctrl_n: std_logic;
-- Registered pattern select, shiftreg. load, counter ena, trigger pulse  
  signal pattern1, pattern0, cntena, trigpulse, trigstart: std_logic;
-- Shiftreg load, shift enable, shiftin, async.reset
  signal loadsreg, gload,  shiften, greset: std_logic;
  signal shift1in, shift2in, shift3in, shift4in: std_logic;
  signal shift1out, shift2out, shift3out, shift4out: std_logic_vector(15 downto 0);
-- Central Counter signals
  signal cout: std_logic_vector(7 downto 0);
-- Multiplexer inpputs,  output (pattern select)
  signal word0, word1, word2, word3, shld: std_logic_vector(15 downto 0);
-- Comparator results; Test outputs
  signal res1, res2, res3, q1in,  q2in, q3in, q1r, q2r, q3r: std_logic;
-- Error latches
  signal l1in, l1out, l2in, l2out, l3in, l3out: std_logic;
-- Parity
  signal par: std_logic_vector(3 downto 0);
-- Link logic signals
  signal linkctena, locktime, hpr: std_logic;
  signal lct: std_logic_vector(7 downto 0);
  
component trreg         -- DFF with TMR
        port( Data: in std_logic; Clock : in  std_logic;
	      Q : out std_logic) ;
end component;

component trreg16       -- 16 bit DFF
        port( Data: in std_logic_vector(15 downto 0); Clock : in  std_logic;
	      Q : out std_logic_vector(15 downto 0)) ;
end component;

component dload         -- Multiplexer 
    port( Data0 : in std_logic_vector(15 downto 0);
          Data1 : in std_logic_vector(15 downto 0);
	  Data2 : in std_logic_vector(15 downto 0);
	  Data3 : in std_logic_vector(15 downto 0);
	  Sel0, Sel1 : in std_logic; 
          Result : out std_logic_vector(15 downto 0)) ;
end component;

component sshift        -- Shiftreg, No TMR, shift left!
    port( Data : in std_logic_vector(15 downto 0);
          Enable, Shiften, Shiftin, Clock : in std_logic;
 	  Q : out std_logic_vector(15 downto 0)) ;
end component;

component trshift       -- Shiftreg with TMR, shift left!
    port( Data : in std_logic_vector(15 downto 0);
          Enable, Shiften, Shiftin, Clock : in std_logic;
	  Q : out std_logic_vector(15 downto 0)) ;
end component;

component compar        -- 16 bit comparator
    port( DataA : in std_logic_vector(15 downto 0);
          DataB : in std_logic_vector(15 downto 0);
	  ANEB : out std_logic);
end component;

component trcnt8        -- 8 bit TMR counter 
    port(Enable, Aclr, Clock : in std_logic;
         Q : out std_logic_vector(7 downto 0)) ;
end component;
	 
component parity        -- 4 input XOR
    port( Data : in std_logic_vector(3 downto 0);
          Result : out std_logic) ;
end component;

---------------------------------------------------------------------
---------------------------------------------------------------------

begin

-- Global Clock and reset
  QCLK <= AC_CLKA;  
  AC_CLKABUF <= not AC_CLKA_IO; -- Inverse clk output

  resetsignal <= not locktime;  -- 4ms from tw pair reset
  rstreg: trreg port map (resetsignal, QCLK, greset); -- greset signal
 
-- G-LINK Chip Configuration  ---------------------------------------
  HP_DIV     <= "00";              -- Highest Baud Rate (700M-1500M)
  HP_FLAGSEL <= '0';               -- Flag is not used  
  HP_FLAG    <= '0';               -- Flag is not used       
  HP_M20SEL  <= '1';               -- 20 bit mode (16data + 4parity)
  HP_MDFSEL  <= '0';               -- Simple Frame Mode
  
-- Pattern Select Inputs (From Twisted Pair) -----------------------
  
  p0: trreg port map (LINE_P0, QCLK, pattern0);   -- Pattern select 
  p1: trreg port map (LINE_P1, QCLK, pattern1);

-- Mux for pattern select ------------------------------------------
  
  ldata: dload port map (word0, word1, word2, word3, 
                         pattern0, pattern1, shld);

  word0 <= "0000000000000001"; -- Walking One
  word1 <= "1111111111111110"; -- Walking Zero
  word2 <= "1010101010101010"; -- Checkerboard
  word3 <= "1111111100000000";
  
-- First shiftreg (No TMR) -----------------------------------------
  firstshift: sshift port map (shld, loadsreg, shiften, shift1in,
                               QCLK, shift1out);

	      shift1in <= shift1out(15); -- Cyclic

	      FBOUT <= shift1out;  -- Feedback output

-- Second shiftreg (With TMR) --------------------------------------
  secondshift: trshift port map (shld, loadsreg, shiften, shift2in,
                                QCLK, shift2out);

	      shift2in <= shift2out(15); -- Cyclic

-- Third shiftreg (With TMR) ---------------------------------------
  thirdshift: trshift port map (shld, loadsreg, shiften, shift3in,
                                QCLK, shift3out);

	      shift3in <= shift3out(15); -- Cyclic

-- Fourth shiftreg (No TMR) ----------------------------------------
  fourthshift: sshift port map (shld, loadsreg, shiften, shift4in,
                               QCLK, shift4out);

	      shift4in <= shift4out(15); -- Cyclic
	      
 shiften <= '1';    -- Active high Shift enable, always enable!
 
-- Comparators -----------------------------------------------------
   -- Result: '1' if not equal	 

  -- First: Between UD input and Second Shiftreg       
  firstcompar: compar port map (FBIN, shift2out, res1);
  
   -- Second: Between Second Shiftreg and 3rd. Shiftreg
  secondcompar: compar port map (shift2out, shift3out, res2);

   -- Third: Between Third Shiftreg and Fourth. Shiftreg
  thirdcompar: compar port map (shift3out, shift4out, res3);
  
-- Error Latches and Test Outputs (Latched comparator output) ------

  errlatch: process (trigstart)
    begin
      if (trigstart='1') then l1in <= '0';     -- Sync. clear after start pulse
	                      l2in <= '0';
			      l3in <= '0';
        else l1in <= res1 or l1out;            -- Store and hold comparator result
	     l2in <= res2 or l2out;
             l3in <= res3 or l3out;
	   
	-- High if not equal and in "test period"
	end if;
    end process errlatch;

  latch1: trreg port map (l1in, QCLK, l1out); -- First test pulses after power on
  latch2: trreg port map (l2in, QCLK, l2out); -- may be low if these registers 
  latch3: trreg port map (l3in, QCLK, l3out); -- power on high 

-- Output latch, Test Outputs  
  outlatch: process(trigstart, gload)
    begin
      if (trigstart='1') then q1in <= not(l1out);  --'0' if Error
	                      q2in <= not(l2out);
 			      q3in <= not(l3out); 
        elsif (gload='1') then q1in <='0';         -- Clear register
	                       q2in <='0';
			       q3in <='0';
	  else q1in <=q1r;                         -- Hold value;
	       q2in <=q2r;
	       q3in <=q3r;
      end if;
  end process outlatch;
  
  q1out: trreg port map (q1in, QCLK, q1r);  Q1 <= q1r;
  q2out: trreg port map (q2in, QCLK, q2r);  Q2 <= q2r;
  q3out: trreg port map (q3in, QCLK, q3r);  Q3 <= q3r;

-- Central counter (Trigger generator) -----------------------------	      

  trigger:  trcnt8 port map (cntena, greset, QCLK, cout);
  cntena <= '1'; -- Always enable
   -- Trigger pulse during "11xxxxxx" count value
  trigpulse <= cout(7) and cout(6); 
  -- Trigger pulse  start at "1100000"
  trigstart <= cout(7) and cout(6) and not (cout(5) or cout(4) or
               cout(3) or cout(2) or cout(1) or cout(0)); 	       
  -- Gload is active when counter is zero <- load shiftregs 
  gload <= greset or (     
           not (cout(7) or cout(6) or cout(5) or cout(4) or
                cout(3) or cout(2) or cout(1) or cout(0)) );
  	   	   
  sregload: trreg port map (gload, QCLK, loadsreg); -- Registered gload

     -- 1/4 QCLK Per. Pulse -> read period!
  triggerout: trreg port map (trigpulse, QCLK, TRIG);
  
-- Parity generator -----------------------------------------------

  pargen3:  parity port map (FBIN(15 downto 12), par(3)); 
  pargen2:  parity port map (FBIN(11 downto 8),  par(2));
  pargen1:  parity port map (FBIN(7  downto 4),  par(1));
  pargen0:  parity port map (FBIN(3  downto 0),  par(0)); 

-- TX Data Output Register (HP_D[15..0] output) --------------------

  prr3: trreg port map (par(3), QCLK, HP_D(19)); -- Parity 
  prr2: trreg port map (par(2), QCLK, HP_D(18));
  prr1: trreg port map (par(1), QCLK, HP_D(17));
  prr0: trreg port map (par(0), QCLK, HP_D(16));
  txr:  trreg16 port map (FBIN, QCLK, HP_D(15 downto 0));
  --               .....  UD,   UCLK 

-- G-LINK data generator -------------------------------------------  

  data_n <=  (trigpulse and cout(5)) or greset; -- Active low for 7/8 time
  
  ctrlword: process(cout, locktime)  -- SOF and EOF CTRL word 
  begin
    if (greset='1') then ctrl_n <='1';  -- Disable during init.
      elsif ( (cout="11100010") or (cout="0000001") ) then
 	ctrl_n <='0';
      else ctrl_n <='1';
    end if;
  end process ctrlword;
   
  -- HP_DAV_N and CAV is delayed with pipeline in order to synchronise
  -- to output test pattern.	      
  gdat:  trreg port map (data_n, QCLK, data_n1);  -- Data word
  gdat1: trreg port map (data_n1, QCLK, writen); -- Pipeline
  
  -- Disable HP_DAV_N during initialisation
  gdat2: trreg port map (writen, QCLK, HP_DAV_N);  -- For Sync. with load
  gctrl: trreg port map (ctrl_n, QCLK, HP_CAV_N);  -- Ctrl word		 

-- Link Init Logic -------------------------------------------------

  -- Registered remote reset
  initreg: trreg port map  (line_rst, QCLK, linereset);
  linkcnt: trcnt8 port map (linkctena, linereset, QCLK, lct);
  
  -- LDOWN_N, write disabled until almost full ("11xxxx") -> locktime='1'
  locktime <= lct(7) and lct(6);  -- Low during lock period (4 ms)
  -- Stop counting when almost full:
  linkctena <= not locktime;  
   -- Active low HP reset until ctout = "00001111" -> > 5 frame:
  hpreset: trreg port map(hpr, QCLK, HP_RST_N);
  hpr <= lct(7) or lct(6) or lct(5) or lct(4) or lct(3);          
  -- Link Up if locked & almost full
  LDOWN_N <= HP_LOCKED and locktime;
  
  LUPLED_N <= not (HP_LOCKED and locktime);  -- Like LDOWN_N
  TESLED_N <= LINE_SIG4;                    -- For test purpose
  
end behavioural;


  


    
